@@ -8,7 +8,7 @@ use warnings;
 use Config::Tiny ();
 use CPAN::Access::AdHoc::Archive;
 use CPAN::Access::AdHoc::Util qw{
-    :carp __expand_distribution_path __guess_media_type
+    :carp __attr __cache __expand_distribution_path __guess_media_type
 };
 use Digest::SHA ();
 use File::HomeDir ();
@@ -18,27 +18,26 @@ use LWP::UserAgent ();
 use LWP::Protocol ();
 use Module::Pluggable::Object;
 use Safe;
+use Scalar::Util qw{ blessed };
 use Text::ParseWords ();
 use URI ();
 
-our $VERSION = '0.000_08';
+our $VERSION = '0.000_18';
 
-my @attributes = (
-    [ config		=> \&_attr_config,	],	# Must be first
-    [ default_cpan_source => \&_attr_default_cpan_source,	],
-    [ __debug		=> \&_attr_literal,	],
-    [ cpan		=> \&_attr_cpan,	],	# Must be last
-);
+# In the following list of attribute names, 'config' must be first
+# because it supplies default values for everything else. 'cpan' must be
+# after 'default_cpan_source' because 'default_cpan_source' determines
+# how the default value of 'cpan' is computed.
+my @attributes = qw{
+    config __debug http_error_handler default_cpan_source cpan
+};
 
 sub new {
     my ( $class, %arg ) = @_;
 
     my $self = bless {}, ref $class || $class;
 
-    foreach my $attr ( @attributes ) {
-	my $name = $attr->[0];
-	$self->$name( delete $arg{$name} );
-    }
+    $self->__init( \%arg );
 
     %arg
 	and __wail( 'Unknown attribute(s): ', join ', ', sort keys %arg );
@@ -46,6 +45,16 @@ sub new {
     return $self;
 }
 
+sub __init {
+    my ( $self, $arg ) = @_;
+
+    foreach my $name ( @attributes ) {
+	$self->$name( delete $arg->{$name} );
+    }
+
+    return $self;
+}
+
 sub corpus {
     my ( $self, $cpan_id ) = @_;
     $cpan_id = uc $cpan_id;
@@ -71,7 +80,7 @@ sub fetch {
     my $rslt = $ua->get( $url );
 
     $rslt->is_success
-	or __wail( "Failed to get $url: ", $rslt->status_line() );
+	or return $self->http_error_handler()->( $self, $path, $rslt );
 
     __guess_media_type( $rslt, $path );
 
@@ -88,12 +97,14 @@ sub fetch {
 sub fetch_author_index {
     my ( $self ) = @_;
 
-    exists $self->{_cache}{author_index}
-	and return $self->{_cache}{author_index};
+    my $cache = $self->__cache();
+    exists $cache->{author_index}
+	and return $cache->{author_index};
 
-    my $author_details = $self->fetch(
-	'authors/01mailrc.txt.gz'
-    )->get_item_content();
+    my $author_details = $self->fetch( 'authors/01mailrc.txt.gz' );
+    _got_archive( $author_details )
+	or return $author_details;
+    $author_details = $author_details->get_item_content();
 
     my $fh = IO::File->new( \$author_details, '<' )
 	or __wail( "Unable to open string reference: $!" );
@@ -101,7 +112,7 @@ sub fetch_author_index {
     my %author_index;
     while ( <$fh> ) {
 	s/ \s+ \z //smx;
-	my ( $kind, $cpan_id, $address ) = Text::ParseWords::parse_line(
+	my ( undef, $cpan_id, $address ) = Text::ParseWords::parse_line(
 	    qr{ \s+ }smx, 0, $_ );
 	( my $name = $address ) =~ s{ \s+ < (.*) > }{}smx;
 	my $mail_addr = $1;
@@ -111,7 +122,7 @@ sub fetch_author_index {
 	};
     }
 
-    return ( $self->{_cache}{author_index} = \%author_index );
+    return ( $cache->{author_index} = \%author_index );
 }
 
 sub fetch_distribution_archive {
@@ -122,50 +133,74 @@ sub fetch_distribution_archive {
 
 sub fetch_distribution_checksums {
     my ( $self, $distribution ) = @_;
+
     $distribution =~ m{ \A ( .* / ) ( [^/]* ) \z }smx
 	or __wail( "Invalid distribution '$distribution'" );
     my ( $dir, $file ) = ( $1, $2 );
+
     $file eq 'CHECKSUMS'
 	and $file = '';
     my $path = __expand_distribution_path( $dir . 'CHECKSUMS' );
     ( $dir = $path ) =~ s{ [^/]* \z }{}smx;
-    $self->{_cache}{checksums}{$dir} ||= _eval_string(
-	$self->fetch( "authors/id/$path" )->get_item_content() );
+
+    my $cache = $self->__cache();
+
+    if ( ! $cache->{checksums}{$dir} ) {
+	my $archive = $self->fetch( "authors/id/$path" );
+	_got_archive( $archive )
+	    or return $archive;
+	$cache->{checksums}{$dir} = _eval_string(
+	    $archive->get_item_content() );
+    }
+
     $file eq ''
-	and return $self->{_cache}{checksums}{$dir};
-    return $self->{_cache}{checksums}{$dir}{$file};
+	and return $cache->{checksums}{$dir};
+    return $cache->{checksums}{$dir}{$file};
 }
 
+# TODO finish implementing error handling. See above, _got_archive().
+
 sub fetch_module_index {
     my ( $self ) = @_;
 
-    exists $self->{_cache}{module_index}
+    my $cache = $self->__cache();
+
+    exists $cache->{module_index}
 	and return wantarray ?
-	    @{ $self->{_cache}{module_index} } :
-	    $self->{_cache}{module_index}[0];
+	    @{ $cache->{module_index} } :
+	    $cache->{module_index}[0];
 
-    my $packages_details = $self->fetch(
-	'modules/02packages.details.txt.gz'
-    )->get_item_content();
+    my ( $meta, %module );
 
-    my $fh = IO::File->new( \$packages_details, '<' )
-	or __wail( "Unable to open string reference: $!" );
+    # The only way this can return undef is if the http_error_handler
+    # returns it. We take that as a request to cache an empty index.
+    if ( my $packages_details = $self->fetch(
+	    'modules/02packages.details.txt.gz' ) ) {
+	$packages_details = $packages_details->get_item_content();
 
-    my $meta = $self->_read_meta( $fh );
+	my $fh = IO::File->new( \$packages_details, '<' )
+	    or __wail( "Unable to open string reference: $!" );
 
-    my %module;
-    while ( <$fh> ) {
-	chomp;
-	my ( $mod, $ver, $pkg ) = split qr{ \s+ }smx;
-##	'undef' eq $ver
-##	    and $ver = undef;
-	$module{$mod} = {
-	    distribution	=> $pkg,
-	    version		=> $ver,
-	};
+	$meta = $self->_read_meta( $fh );
+
+	while ( <$fh> ) {
+	    chomp;
+	    my ( $mod, @info ) = split qr{ \s+ }smx;
+##	    'undef' eq $ver
+##		and $ver = undef;
+	    my ( $pkg, $ver ) = reverse @info;
+	    defined $ver or $ver = 'undef';
+	    $module{$mod} = {
+		distribution	=> $pkg,
+		version		=> $ver,
+	    };
+	}
+
+    } else {
+	$meta = {};
     }
 
-    $self->{_cache}{module_index} = [ \%module, $meta ];
+    $cache->{module_index} = [ \%module, $meta ];
 
     return wantarray ? ( \%module, $meta ) : \%module;
 }
@@ -173,10 +208,11 @@ sub fetch_module_index {
 sub fetch_registered_module_index {
     my ( $self ) = @_;
 
-    exists $self->{_cache}{registered_module_index}
+    my $cache = $self->__cache();
+    exists $cache->{registered_module_index}
 	and return wantarray ?
-	    @{ $self->{_cache}{registered_module_index} } :
-	    $self->{_cache}{registered_module_index}[0];
+	    @{ $cache->{registered_module_index} } :
+	    $cache->{registered_module_index}[0];
 
     my $packages_details = $self->fetch(
 	'modules/03modlist.data.gz'
@@ -197,21 +233,24 @@ sub fetch_registered_module_index {
 
     my $hash = _eval_string( "$reg\nCPAN::Modulelist->data();" );
 
-    $self->{_cache}{registered_module_index} = [ $hash, $meta ];
+    $cache->{registered_module_index} = [ $hash, $meta ];
 
     return wantarray ? ( $hash, $meta ) : $hash;
 }
 
 sub flush {
     my ( $self ) = @_;
-    delete $self->{_cache};
+    delete $self->{'.cache'};
     return $self;
 }
 
 sub indexed_distributions {
     my ( $self ) = @_;
-    $self->{_cache}{indexed_distributions}
-	and return @{ $self->{_cache}{indexed_distributions} };
+
+    my $cache = $self->__cache();
+
+    $cache->{indexed_distributions}
+	and return @{ $cache->{indexed_distributions} };
 
     my $inx = $self->fetch_module_index();
 
@@ -220,34 +259,60 @@ sub indexed_distributions {
 	$pkg{$info->{distribution}}++;
     }
 
-    return @{ $self->{_cache}{indexed_distributions} = [ sort keys %pkg ] };
+    return @{ $cache->{indexed_distributions} = [ sort keys %pkg ] };
 }
 
 # Set up the accessor/mutators. All mutators interpret undef as being a
 # request to restore the default, from the configuration if that exists,
 # or from the configured default code.
 
-foreach my $info ( @attributes ) {
-    my ( $name, $mutator ) = @{ $info };
-    __PACKAGE__->can( $name ) and next;
+__PACKAGE__->__create_accessor_mutators( @attributes );
+
+sub _create_accessor_mutator_helper {
+    my ( $class, $name, $code ) = @_;
+    $class->can( $name )
+	and return;
+    my $full_name = "${class}::$name";
     no strict qw{ refs };
-    *$name = sub {
-	my ( $self, @arg ) = @_;
-	if ( @arg ) {
-	    my $value = $arg[0];
-	    if ( ! defined $value ) {
-		my $config = $self->config();
-		$value = $config->{_}{$name};
-	    }
-	    $self->{$name} = $mutator->( $self, $name, $value );
-	    return $self;
-	} else {
-	    return $self->{$name};
-	}
-    };
+    *$full_name = $code;
+    return;
 }
 
-# Compute the value of the config attribute.
+sub __create_accessor_mutators {
+    my ( $class, @attrs ) = @_;
+    foreach my $name ( @attrs ) {
+	$class->can( $name ) and next;
+	my $full_name = "${class}::$name";
+	$class->_create_accessor_mutator_helper(
+	    "__attr__${name}__validate" => sub { return $_[1] } );
+	$class->_create_accessor_mutator_helper(
+	    "__attr__${name}__post_assignment" => sub { return $_[1] } );
+	no strict qw{ refs };
+	*$full_name = sub {
+	    my ( $self, @arg ) = @_;
+	    my $attr = $self->__attr();
+	    if ( @arg ) {
+		my $value = $arg[0];
+		not defined $value
+		    and 'config' ne $name
+		    and $value = $self->config()->{_}{$name};
+		my $code;
+		not defined $value
+		    and $code = $self->can( "__attr__${name}__default" )
+		    and $value = $code->( $self );
+		$code = $self->can( "__attr__${name}__validate" )
+		    and $value = $code->( $self, $value );
+		$attr->{$name} = $value;
+		$code = $self->can( "__attr__${name}__post_assignment" )
+		    and $code->( $self );
+		return $self;
+	    } else {
+		return $attr->{$name};
+	    }
+	};
+    }
+    return;
+}
 
 {
 
@@ -260,87 +325,88 @@ foreach my $info ( @attributes ) {
     defined $config_dir
 	and $config_path = File::Spec->catfile( $config_dir, $config_file );
 
-    sub _attr_config {
-	my ( $self, $name, $value ) = @_;
-
-	# If the new value is defined, it must be a Config::Tiny object
-	if ( defined $value ) {
-	    ref $value
-		and eval {
-		    $value->isa( 'Config::Tiny' );
-		    1;
-		}
-		or __wail(
-		    "Attribute '$name' must be a Config::Tiny reference" );
-	# If the config file exists, read it
-	} elsif ( defined $config_path && -f $config_path ) {
-	    $value = Config::Tiny->read( $config_path );
-
-	# Otherwise generate an empty configuration
-	} else {
-	    $value = Config::Tiny->new();
-	}
-
-	# Disallow the config key.
-	delete $value->{_}{config};
-
-	return $value;
+    sub __attr__config__default {
+	my ( $self ) = @_;
+	defined $config_path
+	    and -f $config_path
+	    and return Config::Tiny->read( $config_path );
+	return Config::Tiny->new();
     }
 }
 
-# Compute the value of the default_cpan_source attribute
+sub __attr__config__validate {
+    my ( $self, $value ) = @_;
 
-sub _attr_default_cpan_source {
-    my ( $self, $name, $value ) = @_;
-
-    # The rationale of the default order is:
-    # 1) Mini cpan: guaranteed to be local, and since it is non-core,
-    #    the user had to install it, and can be presumed to be using it.
-    # 2) CPAN minus: since it is non-core, the user had to install it,
-    #    and can be presumed to be using it.
-    # 3) CPAN: It is core, but it needs to be set up to be used, and the
-    #    wrapper will detect if it has not been set up.
-    # 4) CPANPLUS: It is core as of 5.10, and works out of the box, so
-    #    we can not presume that the user actually uses it.
-    defined $value
-	or $value = 'CPAN::Mini,cpanm,CPAN,CPANPLUS';
-    $self->_expand_default_cpan_source( $value );
+    my $err = "Attribute 'config' must be a file name or a " .
+	"Config::Tiny reference";
+    if ( ref $value ) {
+	eval {
+	    $value->isa( 'Config::Tiny' );
+	} or __wail( $err );
+    } else {
+	-f $value
+	    or __wail( $err );
+	$value = Config::Tiny->read( $value );
+    }
 
+    delete $value->{_}{config};
     return $value;
 }
 
-# Compute the value of a literal attribute
+# The rationale of the default order is:
+# 1) Mini cpan: guaranteed to be local, and since it is non-core,
+#    the user had to install it, and can be presumed to be using it.
+# 2) CPAN minus: since it is non-core, the user had to install it,
+#    and can be presumed to be using it.
+# 3) CPAN: It is core, but it needs to be set up to be used, and the
+#    wrapper will detect if it has not been set up.
+# 4) CPANPLUS: It is core as of 5.10, and works out of the box, so
+#    we can not presume that the user actually uses it.
+sub __attr__default_cpan_source__default {
+    return 'CPAN::Mini,cpanm,CPAN,CPANPLUS';
+}
 
-sub _attr_literal {
-    my ( $self, $name, $value );
+sub DEFAULT_HTTP_ERROR_HANDLER {
+    my ( $self, $path, $resp ) = @_;
+    my $url = $self->cpan() . $path;
+    __wail( "Failed to get $url: ", $resp->status_line() );
+}
+
+sub __attr__http_error_handler__default {
+    return \&DEFAULT_HTTP_ERROR_HANDLER;
+}
+
+sub __attr__http_error_handler__validate {
+    my ( $self, $value ) = @_;
+    'CODE' eq ref $value
+	or __wail(
+	q{Attribute 'http_error_handler' must be a code reference}
+    );
     return $value;
 }
 
-# Compute the value of the cpan attribute. The actual computation of the
-# default URL is outsourced to _get_default_url(). The attribute needs a
-# trailing slash so we can just slap the path on the end of it.
+sub __attr__cpan__post_assignment {
+    my ( $self ) = @_;
 
-sub _attr_cpan {
-    my ( $self, $name, $value ) = @_;
+    $self->flush();
 
-    defined $value
-	or $value = $self->_get_default_url();
+    return;
+}
+
+sub __attr__cpan__validate {
+    my ( $self, $value ) = @_;
 
     $value = "$value";	# Stringify
     $value =~ s{ (?<! / ) \z }{/}smx;
 
     my $url = URI->new( $value )
 	or _wail( "Bad URL '$value'" );
-    {
-	my $scheme = $url->scheme();
-	my $ua = LWP::UserAgent->new();
-	$url->can( 'authority' )
-	    and LWP::Protocol::implementor( $scheme )
-	    or __wail ( "URL scheme $scheme: is unsupported" );
-    }
     $value = $url;
 
-    $self->flush();
+    my $scheme = $value->scheme();
+    $value->can( 'authority' )
+	and LWP::Protocol::implementor( $scheme )
+	or __wail ( "URL scheme $scheme: is unsupported" );
 
     return $value;
 }
@@ -381,25 +447,31 @@ sub _checksum {
 {
 
     my $search_path = 'CPAN::Access::AdHoc::Default::CPAN';
-    my %defaulter = map {
-	substr( $_, length( $search_path ) + 2 ) => $_
-    } Module::Pluggable::Object->new(
+    my %defaulter = map { (
+        $_ => $_,
+	substr( $_, length( $search_path ) + 2 ) => $_,
+    ) } Module::Pluggable::Object->new(
 	search_path	=> $search_path,
 	inner	=> 0,
 	require	=> 1,
     )->plugins();
 
-    sub _expand_default_cpan_source {
+    sub __attr__default_cpan_source__validate {
 	my ( $self, $value ) = @_;
-	defined $value
-	    or $value = $self->default_cpan_source();
+
+	ref $value
+	    or $value = [ split qr{ \s* , \s* }smx, $value ];
+
+	'ARRAY' eq ref $value
+	    or __wail( q{Attribute 'default_cpan_source' takes an array } .
+	    q{reference or a comma-delimited string} );
 	my @rslt;
-	foreach my $source ( split qr{ \s* , \s* }smx, $value ) {
+	foreach my $source ( @{ $value } ) {
 	    defined( my $class = $defaulter{$source} )
 		or __wail( "Unknown default_cpan_source '$source'" );
 	    push @rslt, $class;
 	}
-	return @rslt;
+	return \@rslt;
     }
 
 }
@@ -410,26 +482,38 @@ sub _eval_string {
     my ( $string ) = @_;
     $string =~ s/ \015? \012 /\n/smxg;
     my $sandbox = Safe->new();
+    $sandbox->permit_only( ':default' );
     my $rslt = $sandbox->reval( $string );
     $@ and __wail( $@ );
     return $rslt;
 }
 
-# Get the repository URL from the first source that actually supplies
-# it. The CPAN::Access::AdHoc::Default::CPAN plug-ins are called in the order
-# specified in the default_cpan_source attribute, and the first source
-# that actually supplies a URL is used. If that source provides a file:
-# URL, the first such is returned. Otherwise the first URL is returned,
-# whatever its scheme. If no URL can be determined, we die.
+# Return the argument if it is a CPAN::Access::AdHoc::Archive; otherwise
+# just return.
 
-sub _get_default_url {
+sub _got_archive {
+    my ( $rtn ) = @_;
+    blessed( $rtn )
+	and $rtn->isa( 'CPAN::Access::AdHoc::Archive' )
+	and return $rtn;
+    return;
+}
+
+# Get the repository URL from the first source that actually supplies
+# it. The CPAN::Access::AdHoc::Default::CPAN plug-ins are called in the
+# order specified in the default_cpan_source attribute, and the first
+# source that actually supplies a URL is used. If that source provides a
+# file: URL, the first such is returned. Otherwise the first URL is
+# returned, whatever its scheme. If no URL can be determined, we die.
+
+sub __attr__cpan__default {
     my ( $self ) = @_;
 
     my $url;
 
     my $debug = $self->__debug();
 
-    foreach my $class ( $self->_expand_default_cpan_source() ) {
+    foreach my $class ( @{ $self->default_cpan_source() } ) {
 
 	my @url_list = $class->get_default()
 	    or next;
@@ -602,15 +686,17 @@ URL can be computed from any source, an exception is thrown.
 =head3 default_cpan_source
 
 When called with no arguments, this method acts as an accessor, and
-returns the current list of default CPAN sources as a comma-delimited
-string.
+returns the current list of default CPAN sources as an array reference.
+B<This is incompatible with version 0.000_08 and before>, where the
+return was a comma-delimited string.
 
 When called with an argument, this method acts as a mutator, and sets
-the list of default CPAN sources. This list is a comma-delimited string,
-and consists of the names of zero or more
-C<CPAN::Access::AdHoc::Default::CPAN::*> classes, with the common
-prefix removed. See the documentation of these classes for more
-information.
+the list of default CPAN sources. This list is either an array reference
+or a comma-delimited string, and consists of the names of zero or more
+C<CPAN::Access::AdHoc::Default::CPAN::*> classes. With either mechanism
+the names of the classes may be passed without the common prefix, which
+will be added back if needed. See the documentation of these classes for
+more information.
 
 If any of the elements in the string does not represent an existing
 C<CPAN::Access::AdHoc::Default::CPAN::> class, an exception is thrown
@@ -620,6 +706,30 @@ If the argument is C<undef>, the default is restored.
 
 The default is C<'CPAN::Mini,cpanm,CPAN,CPANPLUS'>.
 
+=head3 http_error_handler
+
+When called with no arguments, this method acts as an accessor, and
+returns the current HTTP error handler.
+
+When called with an argument, this method acts as a mutator, and sets
+the HTTP error handler. This must be a code reference.
+
+When an HTTP error is encountered, the handler will be called and passed
+three arguments: the C<CPAN::Access::AdHoc> object, the path relative to
+the base URL of the CPAN repository, and the C<HTTP::Response> object.
+Whatever it returns will be returned by the caller.
+
+If the argument is C<undef>, the default is restored.
+
+The default is C<\&CPAN::Access::AdHoc::DEFAULT_HTTP_ERROR_HANDLER>,
+which throws an exception, giving the URL and the HTTP status line. If
+you do not want to code for every error you might encounter, handle the
+uninteresting errors with
+
+ goto &CPAN::Access::AdHoc::DEFAULT_HTTP_ERROR_HANDLER:
+
+This assumes that you have not modified C<@_>.
+
 =head2 Functionality
 
 These methods are what all the rest is in aid of.
@@ -696,6 +806,10 @@ If called in list context, the first return is the index, and the second
 is another hash reference containing the metadata that appears at the
 top of the expanded index file.
 
+If an HTTP error is encountered while fetching the index, normally an
+error is thrown. But if the C<http_error_handler> returns nothing, an
+empty index (and empty index metadata) are returned.
+
 The results of the first fetch are cached; subsequent calls are supplied
 from cache.
 
@@ -777,12 +891,143 @@ This convenience method returns a list of all indexed distributions in
 ASCIIbetical order. This information is derived from the results of
 L<fetch_module_index()|/fetch_module_index>, and is cached.
 
+=head2 Subclass Methods
+
+The following methods exist for the benefit of subclasses, and should
+not be considered part of the public interface. I am willing to make
+this interface public on request, but until the request comes I will
+consider myself at liberty to modify it without notice.
+
+=head3 __attr
+
+This method returns a hash containing all attributes specific to the
+class that makes the call. This hash may be modified, and in fact must
+be to store new attribute values.
+
+=head3 __cache
+
+This method returns a hash containing all values cached by the object.
+This hash may be modified, and in fact must be to cache new values.
+
+=head3 __create_accessor_mutators
+
+ __PACKAGE__->__create_accessor_mutators( @attributes );
+
+This static method creates accessor/mutator methods for the attributes
+named in its argument list. If a subroutine with the same name as an
+attribute exists at the time this method is called, that subroutine is
+assumed to be the accessor/mutator for that attribute.
+
+The methods created by C<__create_accessor_mutators()> have three hooks
+for behavior modification. For any attribute C<whatever>, these are:
+
+=over
+
+=item __attr__whatever__default
+
+ my ( $self, $value ) = @_;
+ my $code;
+ not defined $value
+     and $code = $self->can( '__attr__whatever__default' )
+     and $value = $code->( $self );
+
+This is called when the mutator is passed a new value of C<undef>. Its
+only argument is the invocant. It must return a valid value of the
+attribute.
+
+If a subclass overrides this, the subclass probably should not call
+C<< $self->SUPER::__attr__whatever__default() >>.
+
+=item __attr__whatever__validate
+
+ my ( $self, $value ) = @_;
+ my $code;
+ $code = $self->can( '__attr__whatever__validate' )
+     and $value = $code->( $self, $value );
+
+This method is called after C<__attr__whatever__default()>, and
+validates the value. It rejects a value by throwing an exception. The
+preferred way to do this is by calling C<__wail()>.
+
+If a subclass overrides this, the subclass B<must> execute
+
+ $value = $self->SUPER::__attr__whatever__validate( $value );
+
+before it performs its own validation. The superclass method B<must>
+return the internal format of the attribute's value, which the subclass
+B<must> return after validating.
+
+=item __attr__whatever__post_assignment
+
+ $self->__attr__whatever__post_assignment()
+
+This method is called after the new value has been assigned to the
+attribute.
+
+If a subclass overrides this, it B<must> call
+C<< $self->SUPER::__attr__whatever__post_assignment() >>, and it
+B<should> call it last thing before returning.
+
+=back
+
+All these hooks are optional, but C<__create_accessor_mutators()> will
+generate dummy C<__attr__whatever__validate()> and
+C<__attr__whatever__post_assignment()> methods for any attributes that
+do not have them at the time it is called.
+
+=head3 __init
+
+This method is called when a new object is instantiated. Its arguments
+are the invocant and a reference to a hash containing attribute names
+and values.
+
+If a subclass adds attributes, it B<must> override this method. The
+override B<must> call C<< $self->SUPER::__init( $args ) >> first thing.
+It B<must> then set its own attributes from the C<$args> hash reference,
+deleting them from the hash. The override returns nothing.
+
 =head1 SEE ALSO
 
-L<CPAN::DistnameInfo|CPAN::DistnameInfo>, which parses distribution name
-and version (among other things) from the name of a particular
-distribution archive. This was very helpful in some of my CPAN
-ad-hocery.
+L<App::cpanlistchanges|App::cpanlistchanges> by Tatsuhiko Miaygawa lists
+Changes files -- by default the changes from the version you have
+installed to the most-current CPAN version.
+
+L<CPAN::DistnameInfo|CPAN::DistnameInfo> by Graham Barr, which parses
+distribution name and version (among other things) from the name of a
+particular distribution archive. This was very helpful in some of my
+CPAN ad-hocery.
+
+L<CPAN::Easy|CPAN::Easy> by Chris Weyl, which retrieves distributions
+and their meta information. As of this writing, it does not support
+version 2.0 of the meta spec.
+
+L<CPAN::Index|CPAN::Index> by Adam Kennedy, which accesses the CPAN
+indices, storing them in an SQLite database.
+
+L<CPAN::Inject|CPAN::Inject> by Adam Kennedy, which injects tarballs
+into a F<.cpan/sources> directory for a given CPAN ID.
+
+L<CPAN::Meta|CPAN::Meta> by David Golden, which presents a unified
+interface for the various versions of the CPAN meta-data specification.
+
+L<CPAN::Mini|CPAN::Mini> by Ricardo Signes, which lets you have your own
+personal CPAN, optionally with only the latest distributions.
+
+L<CPAN::Mini::Devel|CPAN::Mini::Devel> by David Golden, which is
+L<CPAN::Mini|CPAN::Mini> with the addition of developer releases.
+
+L<CPAN::Mini::Inject|CPAN::Mini::Inject> by Christian Walde, which
+injects distributions into a Mini-CPAN.
+
+L<CPAN::PackageDetails|CPAN::PackageDetails> by Brian D Foy, which reads
+and writes the CPAN F<modules/02packages.details.txt.gz> file.
+
+L<Parse::CPAN::Packages|Parse::CPAN::Packages> by Christian Wade, which
+parses the CPAN F<modules/02packages.details.txt.gz> file.
+
+L<Parse::CPAN::Packages::Fast|Parse::CPAN::Packages::Fast> by Slaven
+Rezic, non-OO code which parses the CPAN
+F<modules/02packages.details.txt.gz> file.
 
 =head1 SUPPORT
 
@@ -795,7 +1040,7 @@ Thomas R. Wyant, III F<wyant at cpan dot org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2012 by Thomas R. Wyant, III
+Copyright (C) 2012-2013 by Thomas R. Wyant, III
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl 5.10.0. For more details, see the full text
