@@ -14,7 +14,7 @@ use HTTP::Response ();
 use Module::Pluggable::Object;
 use URI::file;
 
-our $VERSION = '0.000_08';
+our $VERSION = '0.000_18';
 
 # Note that this can be called as a mutator, but the mutator
 # functionality is private to the invocant's class.
@@ -48,20 +48,6 @@ sub get_item_mtime {
     __weep( 'The get_item_mtime() method must be overridden' );
 }
 
-sub guess_media_type {
-    my ( $class, $resp, $path ) = @_;
-
-    __whinge( join ' ',
-	'CPAN::Access::AdHoc::Archive::guess_media_type() is',
-	'deprecated in favor of',
-	'CPAN::Access::AdHoc::Util::__guess_media_type()'
-    );
-
-    __guess_media_type( $resp, $path );
-
-    return;
-}
-
 {
     my @archivers = Module::Pluggable::Object->new(
 	search_path	=> 'CPAN::Access::AdHoc::Archive',
@@ -69,14 +55,6 @@ sub guess_media_type {
 	require	=> 1,
     )->plugins();
 
-    sub handle_http_response {
-	__whinge( join ' ',
-	    'handle_http_response() is deprecated in favor of',
-	    '__handle_http_response()',
-	);
-	goto &__handle_http_response;
-    }
-
     sub __handle_http_response {
 	my ( $class, $resp ) = @_;
 
@@ -124,6 +102,22 @@ sub metadata {
 
 }
 
+# Note that this can be called as a mutator, but the mutator
+# functionality is private to the invocant's class.
+sub mtime {
+    my ( $self, @value ) = @_;
+    my $attr = $self->__attr();
+
+    if ( @value ) {
+	caller eq ref $self
+	    or __wail( 'Attribute archive is read-only' );
+	$attr->{mtime} = $value[0];
+	return $self;
+    } else {
+	return $attr->{mtime};
+    }
+}
+
 # Note that this can be called as a mutator, but the mutator
 # functionality is private to the invocant's class.
 sub path {
@@ -141,7 +135,9 @@ sub path {
 }
 
 sub wrap_archive {
-    my ( $self, $fn, $author_dir ) = @_;
+    my ( $class, @args ) = @_;
+    my $opt = 'HASH' eq ref $args[0] ? shift @args : {};
+    my ( $fn ) = @args;
     -f $fn
 	or __wail( "File $fn not found" );
     my $content;
@@ -153,12 +149,15 @@ sub wrap_archive {
 	close $fh;
     }
     my $path;
-    if ( 'SCALAR' eq ref $author_dir ) {
-	$path = ${ $author_dir };
+    if ( defined $opt->{directory} ) {
+	defined $opt->{author}
+	    and __wail(
+	    q{Specifying both 'author' and 'directory' is ambiguous} );
+	$path = $opt->{directory};
 	$path =~ s{ (?<! / ) \z }{/}smx;
 	$path .= ( File::Spec->splitpath( $fn ) )[2];
-    } elsif ( defined $author_dir ) {
-	my $author_path = __expand_distribution_path( $author_dir );
+    } elsif ( defined $opt->{author} ) {
+	my $author_path = __expand_distribution_path( $opt->{author} );
 	$author_path =~ s{ / \z }{}smx;
 	$path = join '/', 'authors/id', $author_path,
 	    ( File::Spec->splitpath( $fn ) )[2];
@@ -166,11 +165,19 @@ sub wrap_archive {
 	my $uri = URI::file->new( Cwd::abs_path( $fn ) );
 	$path = $uri->as_string();
 	$path =~ s{ \A .* / (?= authors/ | modules/ ) }{}smx
-	    or $path =~ s{ \A [^/]* // }{}smx;
+	    or do {
+	    my @parts = File::Spec->splitpath( $uri->file() );
+	    my @dir = File::Spec->splitdir( $parts[1] );
+	    $dir[-1] eq ''
+		and pop @dir;
+	    my $author_path = __expand_distribution_path( $dir[-1] );
+	    $author_path =~ s{ / \z }{}smx;
+	    $path = join '/', 'authors/id', $author_path, $parts[2];
+	};
     }
     my $resp = HTTP::Response->new( 200, 'OK', undef, $content );
     __guess_media_type( $resp, $path );
-    return $self->__handle_http_response( $resp );
+    return $class->__handle_http_response( $resp );
 }
 
 sub write : method {	## no critic (ProhibitBuiltInHomonyms)
@@ -191,24 +198,13 @@ This class is not intended to be used directly.
 
 =head1 NOTICE
 
-Effective with version 0.000_06:
+Effective with version 0.000_12:
 
-* Static method C<__guess_media_type()> is deprecated. The code has been
-moved to C<CPAN::Access::AdHoc::Util> as subroutine
-C<__guess_media_type()>, which is private to the C<CPAN-Access-AdHoc>
-package.
-
-* Static method C<handle_http_response()> is deprecated. The code has
-been renamed to C<__handle_http_response()>, and is considered private
-to the C<CPAN-Access-AdHoc> package.
-
-Because the deprecated methods have never been in a production release,
-they will be removed a week after the publication of version 0.000_06.
-
-I have never been real happy about exposing these in the public
-interface, and with the writing of
-C<< CPAN::Access::AdHoc::Archive->wrap_archive() >>, the need to expose
-them appears to me to have gone away.
+Method C<wrap_archive()> takes an optional leading hash. You can use
+either key C<{author}> to specify the CPAN author ID for the archive, or
+key C<{directory}> to specify its archive relative to the CPAN root. The
+argument after the file name is deprecated, and will be removed a week
+after the publication of 0.000_12.
 
 =head1 DESCRIPTION
 
@@ -262,6 +258,10 @@ Subclasses are expected to support encodings C<'gzip'> and C<'x-bzip2'>.
 Again, nothing good will happen if the content is not actually encoded
 this way.
 
+=item mtime
+
+This is the modification time of the archive.
+
 =item path
 
 This optional argument is intended to contain the path to the archive.
@@ -283,8 +283,15 @@ These methods retrieve or modify the attributes of the class.
 =head3 archive
 
 This method is an accessor for the object representing the archive that
-actually contains the CPAN distribution. This attribute is read-only, so
-it is an error to pass an argument.
+actually contains the CPAN distribution.
+
+This attribute is read-only, so it is an error to pass an argument.
+
+=head3 mtime
+
+This method is an accessor for the time the archive was last modified.
+
+This attribute is read-only, so it is an error to pass an argument.
 
 =head3 path
 
@@ -325,21 +332,6 @@ This method returns the modification time of the named item in the
 archive. The name of the item is specified relative to
 C<< $arc->base_directory() >>.
 
-=head3 guess_media_type
-
- CPAN::Access::AdHoc::Archive->guess_media_type( $resp, $path );
-
-This static method is deprecated. It is a wrapper for
-C<CPAN::Access::AdHoc::__guess_media_type()>.
-
-Because this method has never appeared in a production release, it will
-be removed a week after the next release, which will be a development
-release.
-
-=head3 handle_http_response
-
-This static method is deprecated in favor of __handle_http_response().
-
 =head3 __handle_http_response
 
 This static method is private to the C<CPAN-Access-AdHoc> package.
@@ -353,7 +345,7 @@ Otherwise, it simply returns.
 The method can do anything it wants to evaluate its argument, but
 typically it examines the C<Content-Type>, C<Content-Encoding>, and
 C<Content-Location> headers. The expected values of these headers are
-those loaded by C<LWP::MediaTypes::guess_media_type()>.
+those loaded by C<CPAN::Access::AdHoc::Util::__guess_media_type()>.
 
 For this class (i.e. C<CPAN::Access::AdHoc::Archive>), the method simply
 calls C<__handle_http_response()> on all the
@@ -363,11 +355,6 @@ C<CPAN::Access::AdHoc::Archive> object. If none of the subclasses
 handles the L<HTTP::Response|HTTP::Response> object, nothing is
 returned.
 
-The whole C<guess_media_type()>/C<__handle_http_response()> thing seems
-like a crock to me, but I have not been able to think of anything
-better. If they make it into a production release, they B<will> go
-through a deprecation cycle.
-
 =head3 item_present
 
  $arc->item_present( 'Build.PL' )
@@ -393,21 +380,28 @@ method makes no further effort to establish what the metadata are.
 =head3 wrap_archive
 
  my $archive = CPAN::Access::AdHoc::Archive->wrap_archive(
-     'foo/MyDistrib-0.001.tar.gz', 'MYCPANID' );
+     { author => 'MYCPANID' },
+     'foo/MyDistrib-0.001.tar.gz' );
 
 This method (either normal or static) makes a
 C<CPAN::Access::AdHoc::Archive> object out of a local file, and returns
 it.
 
-The second argument specifies the path to the item in a CPAN archive.
-This is combined with the base name of the file and the result is used
-to populate the C<path> attribute of the archive. A string specifies a
-CPAN ID directory; this will be expanded and C<'authors/id/'> prepended.
-A string reference specifies a literal path from the archive root. An
-undefined (or omitted) value causes the full path name of the file to be
-made into a C<file:> URL, and any prefix before C</authors/> or
-C</modules/> is removed; if these are not found, the full path name of
-the file is used.
+The leading hash before the file name is optional, and specifies the
+path to the item in a CPAN archive. This is combined with the base name
+of the file and the result is used to populate the C<path> attribute of
+the archive.  Possible keys are
+
+ author => the CPAN author ID for the archive;
+ directory => The directory relative to the CPAN archive root.
+
+You may not specify both C<author> and C<directory>, since this is
+ambiguous.
+
+If no option hash is specified, or neither C<author> nor C<directory> is
+specified in it, the full path name will be made into a C<file:> URL,
+and any prefix before C</authors/> or C</modules/> is removed; if these
+are not found, the full path name of the file is used.
 
 =head3 write
 
@@ -434,7 +428,7 @@ Thomas R. Wyant, III F<wyant at cpan dot org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2012 by Thomas R. Wyant, III
+Copyright (C) 2012-2013 by Thomas R. Wyant, III
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl 5.10.0. For more details, see the full text
