@@ -10,11 +10,12 @@ use base qw{ CPAN::Access::AdHoc::Archive };
 use Archive::Tar ();
 use CPAN::Access::AdHoc::Util qw{ :carp __guess_media_type };
 use File::Spec::Unix ();
+use HTTP::Date ();
 use IO::File ();
 use IO::Uncompress::Bunzip2 ();
 use IO::Uncompress::Gunzip ();
 
-our $VERSION = '0.000_08';
+our $VERSION = '0.000_18';
 
 {
 
@@ -57,6 +58,7 @@ our $VERSION = '0.000_08';
 
 	}
 
+	$self->mtime( delete $arg{mtime} );
 	$self->path( delete $arg{path} );
 
 	return $self;
@@ -114,14 +116,6 @@ sub get_item_mtime {
 
     my %handled = map { $_ => 1 } qw{ application/x-tar };
 
-    sub handle_http_response {
-	__whinge( join ' ',
-	    'handle_http_response() is deprecated in favor of',
-	    '__handle_http_response()',
-	);
-	goto &__handle_http_response;
-    }
-
     sub __handle_http_response {
 	my ( $class, $rslt ) = @_;
 
@@ -131,6 +125,8 @@ sub get_item_mtime {
 	return $class->new(
 	    content	=> \( scalar $rslt->content() ),
 	    encoding	=> scalar $rslt->header( 'Content-Encoding' ),
+	    mtime	=> HTTP::Date::str2time(
+		scalar $rslt->header( 'Last-Modified' ) ),
 	    path	=> scalar $rslt->header( 'Content-Location' ),
 	);
     }
@@ -281,7 +277,7 @@ Thomas R. Wyant, III F<wyant at cpan dot org>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2012 by Thomas R. Wyant, III
+Copyright (C) 2012-2013 by Thomas R. Wyant, III
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl 5.10.0. For more details, see the full text
